==> group_subscription-2.0.0-alpha3.tar.gz: group_subscription/src/Plugin/QueueWorker/SubscriptionBuilderBase.php <==
<?php

/**
 * @file
 * Contains Drupal\group_subscription\Plugin\QueueWorker\SubscriptionBuilderBase.php
 */

namespace Drupal\group_subscription\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Queue\SuspendQueueException;
use Drupal\Core\Database\Database;

if (!defined('GROUP_SUBSCRIPTION_TYPE_DISABLED')) {
  define('GROUP_SUBSCRIPTION_TYPE_DISABLED', 0);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE')) {
  define('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE', 1);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_DAILY')) {
  define('GROUP_SUBSCRIPTION_TYPE_DAILY', 2);
}

/**
 * Provides base functionality for the Subscription Builder Queue Workers.
 */
abstract class SubscriptionBuilderBase extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * State API object.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The node storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $nodeStorage;

  /**
   * The comment storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $commentStorage;

  /**
   * The group storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $groupStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
      StateInterface $state,
      EntityStorageInterface $node_storage,
      EntityStorageInterface $comment_storage,
      EntityStorageInterface $group_storage
  ) {
    $this->state = $state;
    $this->nodeStorage = $node_storage;
    $this->commentStorage = $comment_storage;
    $this->groupStorage = $group_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('state'),
      $container->get('entity.manager')->getStorage('node'),
      $container->get('entity.manager')->getStorage('comment'),
      $container->get('entity.manager')->getStorage('group')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $activity = $data;

    // Check daily subscription queue status.
    $queue_start = $this->state->get('group_subscription.daily_queue_start');
    $queue_start = (is_numeric($queue_start)) ? $queue_start : 0;
    $queue_end = $this->state->get('group_subscription.daily_queue_end');
    $queue_end = (is_numeric($queue_end)) ? $queue_end : 0;

    // Process items if we're not in the middle of sending daily subscriptions.
    if ($queue_start <= $queue_end) {
      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');

      // Be sure group specified in $data still exists.
      $group = $this->groupStorage->load($data->gid);
      if (is_null($group)) {
        // This group does not exist.
        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Subscription builder cannot process group. The group ID %gid cannot be found. Processing for %entitytype with ID %entityid will be skipped.',
            ['%gid' => $data->gid, '%entitytype' => $data->entitytype, '%entityid' => $data->entityid]);
        }
        return;
      }

      // Be sure entity specified in $data still exists.
      if ($data->entitytype === 'comment') {
        $comment = $this->commentStorage->load($data->entityid);
        if (is_null($comment) || !$comment->isPublished()) {
          // This comment does not exist or has been unpublished.
          if ($logging) {
            \Drupal::logger('group_subscription')->notice('Subscription builder cannot process %entitytype. The %entitytype with ID %entityid is either unpublished cannot be found. Processing for this item on %parenttype with ID %parentid in group ID %gid will be skipped.',
              ['%entitytype' => $data->entitytype, '%entityid' => $data->entityid, '%parenttype' => $data->parenttype, '%parentid' => $data->parentid, '%gid' => $data->gid]);
          }
          return;
        }
      }
      else {
        $entity = $this->nodeStorage->load($data->entityid);
        if (is_null($entity) || !$entity->isPublished()) {
          // This entity does not exist or has been unpublished.
          if ($logging) {
            \Drupal::logger('group_subscription')->notice('Subscription builder cannot process %entitytype. The %entitytype with ID %entityid is either unpublished or cannot be found. Processing for this item in group ID %gid will be skipped.',
              ['%entitytype' => $data->entitytype, '%entityid' => $data->entityid, '%gid' => $data->gid]);
          }
          return;
        }
      }

      // See if a hold exists for this item.
      $held = \Drupal::database()->select('group_subscription_held', 'g')
        ->fields('g', ['entityid', 'entitytype'])
        ->condition('gid', $data->gid)
        ->condition('entityid', $data->entityid)
        ->condition('entitytype', $data->entitytype)
        ->execute()->fetchObject();

      // If no hold exists build subscriptions.
      if (!$held) {
        // Add to group subscription immediate queue.
        $this->addToImmediate($activity);

        // Add to group subscription daily queue.
        $this->addToDaily($activity);
      }
    }
    else {
      // There are items to build but we're processing daily subscription items.
      // Release this item and wait for daily subscriptions to finish sending.
      throw new SuspendQueueException('Group subscription builder waiting for daily subscription sender to complete.');
    }
  }

  /**
   * Create an immediate email for each user subscribed to this activity group.
   *
   * @return int
   */
  protected function addToImmediate($activity) {
    // Get immediate subscriptions for this group for active users with master switch on.
    $query = \Drupal::database()->select('group_subscription_groups', 'g');
    $query->join('group_subscription_global', 'gl', 'g.uid = gl.uid');
    $query->join('users_field_data', 'f', 'g.uid = f.uid');
    $query->fields('g', ['uid'])
      ->condition('g.gid', $activity->gid)
      ->condition('g.subscription', GROUP_SUBSCRIPTION_TYPE_IMMEDIATE)
      ->condition('gl.status', 1)
      ->condition('f.status', 1);

    $users = $query->execute()->fetchAll();
    if (count($users)) {
      // Create a new class to hold the email data.
      $email = new \stdClass();

      // Grab the immediate email queue.
      $queue_factory = \Drupal::service('queue');
      $email_queue = $queue_factory->get('cron_subscription_immediate');

      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');
      $create_count = 0;

      // For each immediate subscription with a group id matching this activity.
      foreach ($users as $user) {
        // Create the email object to be processed.
        $email->uid = $user->uid;
        $email->gid = $activity->gid;
        $email->entityid = $activity->entityid;
        $email->entitytype = $activity->entitytype;
        $email->parentid = $activity->parentid;
        $email->parenttype = $activity->parenttype;
        $email->bundle = $activity->bundle;

        // Add the email object to the queue.
        $created = $email_queue->createItem($email);

        if ($created && $logging) {
          $create_count++;
        }
      }

      if ($logging) {
        \Drupal::logger('group_subscription')->notice('Immediate builder created %createcount out of %attemptcount immediate email subscriptions for the %entitytype with entityid: %entityid in groupid: %gid.',
          ['%createcount' => $create_count, '%attemptcount' => count($users), '%entitytype' => $activity->entitytype, '%entityid' => $activity->entityid, '%gid' => $activity->gid]);
      }
    }
  }

  /**
   * Create/update a daily email for each user subscribed to this activity group.
   *
   * @return int
   */
  protected function addToDaily($activity) {
    // Get daily subscriptions for this group for active users with master switch on.
    $query = \Drupal::database()->select('group_subscription_groups', 'g');
    $query->join('group_subscription_global', 'gl', 'g.uid = gl.uid');
    $query->join('users_field_data', 'f', 'g.uid = f.uid');
    $query->fields('g', ['uid'])
      ->condition('g.gid', $activity->gid)
      ->condition('g.subscription', GROUP_SUBSCRIPTION_TYPE_DAILY)
      ->condition('gl.status', 1)
      ->condition('f.status', 1);

    $users = $query->execute()->fetchCol();
    if (count($users)) {
      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');
      $create_count = 0;

      // See if the entityid already exists in the activity table.
      $query = \Drupal::database()->select('group_subscription_activity_daily', 'g');
      $query->fields('g', ['uid'])
        ->condition('g.entityid', $activity->entityid)
        ->condition('g.entitytype', $activity->entitytype);
      $existing = $query->execute()->fetchAll();
      $activity_count = count($existing);

      // If the entityid cannot be found in the activity table add records.
      if ($activity_count === 0) {
        // Field list for database insert.
        $fields = [
          'uid',
          'gid',
          'entityid',
          'entitytype',
          'parentid',
          'parenttype',
          'bundle',
          'timestamp',
        ];

        // Begin creating insert query.
        $insert = \Drupal::database()->insert('group_subscription_activity_daily', $options = ['return' => Database::RETURN_AFFECTED])
          ->fields($fields);

        // Cycle through all users to create a single insert query.
        foreach ($users as $user) {
          $data = [
            $user,
            $activity->gid,
            $activity->entityid,
            $activity->entitytype,
            $activity->parentid,
            $activity->parenttype,
            $activity->bundle,
            time(),
          ];

          $insert->values(array_combine($fields, $data));
        }

        $create_count = $insert->execute();

        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Daily builder created %createcount out of %attemptcount daily activity records for the %entitytype with entityid: %entityid in groupid: %gid.',
            ['%createcount' => $create_count, '%attemptcount' => count($users), '%entitytype' => $activity->entitytype, '%entityid' => $activity->entityid, '%gid' => $activity->gid]);
        }
      }
      else {
        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Daily builder attempted to create %attemptcount daily activity records for the %entitytype with entityid: %entityid in groupid: %gid but %activitycount rows matching this entityid were already found.',
            ['%attemptcount' => count($users), '%entitytype' => $activity->entitytype, '%entityid' => $activity->entityid, '%gid' => $activity->gid, '%activitycount' => $activity_count]);
        }
      }
    }
  }
}

==> group_subscription-3.0.0-alpha3.tar.gz: group_subscription/src/Plugin/QueueWorker/SubscriptionBuilderBase.php <==
<?php

namespace Drupal\group_subscription\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Queue\SuspendQueueException;
use Drupal\Core\Database\Database;

if (!defined('GROUP_SUBSCRIPTION_TYPE_DISABLED')) {
  define('GROUP_SUBSCRIPTION_TYPE_DISABLED', 0);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE')) {
  define('GROUP_SUBSCRIPTION_TYPE_IMMEDIATE', 1);
}

if (!defined('GROUP_SUBSCRIPTION_TYPE_DAILY')) {
  define('GROUP_SUBSCRIPTION_TYPE_DAILY', 2);
}

/**
 * Provides base functionality for the Subscription Builder Queue Workers.
 */
abstract class SubscriptionBuilderBase extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * State API object.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The node storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $nodeStorage;

  /**
   * The comment storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $commentStorage;

  /**
   * The group storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $groupStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
      StateInterface $state,
      EntityStorageInterface $node_storage,
      EntityStorageInterface $comment_storage,
      EntityStorageInterface $group_storage
  ) {
    $this->state = $state;
    $this->nodeStorage = $node_storage;
    $this->commentStorage = $comment_storage;
    $this->groupStorage = $group_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('state'),
      $container->get('entity_type.manager')->getStorage('node'),
      $container->get('entity_type.manager')->getStorage('comment'),
      $container->get('entity_type.manager')->getStorage('group')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $activity = $data;

    // Check daily subscription queue status.
    $queue_start = $this->state->get('group_subscription.daily_queue_start');
    $queue_start = (is_numeric($queue_start)) ? $queue_start : 0;
    $queue_end = $this->state->get('group_subscription.daily_queue_end');
    $queue_end = (is_numeric($queue_end)) ? $queue_end : 0;

    // Process items if we're not in the middle of sending daily subscriptions.
    if ($queue_start <= $queue_end) {
      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');

      // Be sure group specified in $data still exists.
      $group = $this->groupStorage->load($data->gid);
      if (is_null($group)) {
        // This group does not exist.
        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Subscription builder cannot process group. The group ID %gid cannot be found. Processing for %entitytype with ID %entityid will be skipped.',
            [
              '%gid' => $data->gid,
              '%entitytype' => $data->entitytype,
              '%entityid' => $data->entityid,
            ]);
        }
        return;
      }

      // Be sure entity specified in $data still exists.
      if ($data->entitytype === 'comment') {
        $comment = $this->commentStorage->load($data->entityid);
        if (is_null($comment) || !$comment->isPublished()) {
          // This comment does not exist or has been unpublished.
          if ($logging) {
            \Drupal::logger('group_subscription')->notice('Subscription builder cannot process %entitytype. The %entitytype with ID %entityid is either unpublished cannot be found. Processing for this item on %parenttype with ID %parentid in group ID %gid will be skipped.',
              [
                '%entitytype' => $data->entitytype,
                '%entityid' => $data->entityid,
                '%parenttype' => $data->parenttype,
                '%parentid' => $data->parentid,
                '%gid' => $data->gid,
              ]);
          }
          return;
        }
      }
      else {
        $entity = $this->nodeStorage->load($data->entityid);
        if (is_null($entity) || !$entity->isPublished()) {
          // This entity does not exist or has been unpublished.
          if ($logging) {
            \Drupal::logger('group_subscription')->notice('Subscription builder cannot process %entitytype. The %entitytype with ID %entityid is either unpublished or cannot be found. Processing for this item in group ID %gid will be skipped.',
              [
                '%entitytype' => $data->entitytype,
                '%entityid' => $data->entityid,
                '%gid' => $data->gid,
              ]);
          }
          return;
        }
      }

      // See if a hold exists for this item.
      $held = \Drupal::database()->select('group_subscription_held', 'g')
        ->fields('g', ['entityid', 'entitytype'])
        ->condition('gid', $data->gid)
        ->condition('entityid', $data->entityid)
        ->condition('entitytype', $data->entitytype)
        ->execute()->fetchObject();

      // If no hold exists build subscriptions.
      if (!$held) {
        // Add to group subscription immediate queue.
        $this->addToImmediate($activity);

        // Add to group subscription daily queue.
        $this->addToDaily($activity);
      }
    }
    else {
      // There are items to build but we're processing daily subscription items.
      // Release this item and wait for daily subscriptions to finish sending.
      throw new SuspendQueueException('Group subscription builder waiting for daily subscription sender to complete.');
    }
  }

  /**
   * Create an immediate email for each user subscribed to this activity group.
   */
  protected function addToImmediate($activity) {
    // Get immediate subs for this group for active users with master switch on.
    $query = \Drupal::database()->select('group_subscription_groups', 'g');
    $query->join('group_subscription_global', 'gl', 'g.uid = gl.uid');
    $query->join('users_field_data', 'f', 'g.uid = f.uid');
    $query->fields('g', ['uid'])
      ->condition('g.gid', $activity->gid)
      ->condition('g.subscription', GROUP_SUBSCRIPTION_TYPE_IMMEDIATE)
      ->condition('gl.status', 1)
      ->condition('f.status', 1);

    $users = $query->execute()->fetchAll();
    if (count($users)) {
      // Create a new class to hold the email data.
      $email = new \stdClass();

      // Grab the immediate email queue.
      $queue_factory = \Drupal::service('queue');
      $email_queue = $queue_factory->get('cron_subscription_immediate');

      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');
      $create_count = 0;

      // For each immediate subscription with a group id matching this activity.
      foreach ($users as $user) {
        // Create the email object to be processed.
        $email->uid = $user->uid;
        $email->gid = $activity->gid;
        $email->entityid = $activity->entityid;
        $email->entitytype = $activity->entitytype;
        $email->parentid = $activity->parentid;
        $email->parenttype = $activity->parenttype;
        $email->bundle = $activity->bundle;

        // Add the email object to the queue.
        $created = $email_queue->createItem($email);

        if ($created && $logging) {
          $create_count++;
        }
      }

      if ($logging) {
        \Drupal::logger('group_subscription')->notice('Immediate builder created %createcount out of %attemptcount immediate email subscriptions for the %entitytype with entityid: %entityid in groupid: %gid.',
          [
            '%createcount' => $create_count,
            '%attemptcount' => count($users),
            '%entitytype' => $activity->entitytype,
            '%entityid' => $activity->entityid,
            '%gid' => $activity->gid,
          ]);
      }
    }
  }

  /**
   * Create/update a daily email for each user subscribed to this activity group.
   */
  protected function addToDaily($activity) {
    // Get daily subs for this group for active users with master switch on.
    $query = \Drupal::database()->select('group_subscription_groups', 'g');
    $query->join('group_subscription_global', 'gl', 'g.uid = gl.uid');
    $query->join('users_field_data', 'f', 'g.uid = f.uid');
    $query->fields('g', ['uid'])
      ->condition('g.gid', $activity->gid)
      ->condition('g.subscription', GROUP_SUBSCRIPTION_TYPE_DAILY)
      ->condition('gl.status', 1)
      ->condition('f.status', 1);

    $users = $query->execute()->fetchCol();
    if (count($users)) {
      // Prepare detailed logging.
      $config_factory = \Drupal::configFactory();
      $config = $config_factory->getEditable('group_subscription.settings');
      $logging = $config->get('group_subscription_watchdog');
      $create_count = 0;

      // See if the entityid already exists in the activity table.
      $query = \Drupal::database()->select('group_subscription_activity_daily', 'g');
      $query->fields('g', ['uid'])
        ->condition('g.entityid', $activity->entityid)
        ->condition('g.entitytype', $activity->entitytype);
      $existing = $query->execute()->fetchAll();
      $activity_count = count($existing);

      // If the entityid cannot be found in the activity table add records.
      if ($activity_count === 0) {
        // Field list for database insert.
        $fields = [
          'uid',
          'gid',
          'entityid',
          'entitytype',
          'parentid',
          'parenttype',
          'bundle',
          'timestamp',
        ];

        // Begin creating insert query.
        $insert = \Drupal::database()->insert('group_subscription_activity_daily', $options = ['return' => Database::RETURN_AFFECTED])
          ->fields($fields);

        // Cycle through all users to create a single insert query.
        foreach ($users as $user) {
          $data = [
            $user,
            $activity->gid,
            $activity->entityid,
            $activity->entitytype,
            $activity->parentid,
            $activity->parenttype,
            $activity->bundle,
            time(),
          ];

          $insert->values(array_combine($fields, $data));
        }

        $create_count = $insert->execute();

        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Daily builder created %createcount out of %attemptcount daily activity records for the %entitytype with entityid: %entityid in groupid: %gid.',
            [
              '%createcount' => $create_count,
              '%attemptcount' => count($users),
              '%entitytype' => $activity->entitytype,
              '%entityid' => $activity->entityid,
              '%gid' => $activity->gid,
            ]);
        }
      }
      else {
        if ($logging) {
          \Drupal::logger('group_subscription')->notice('Daily builder attempted to create %attemptcount daily activity records for the %entitytype with entityid: %entityid in groupid: %gid but %activitycount rows matching this entityid were already found.',
            [
              '%attemptcount' => count($users),
              '%entitytype' => $activity->entitytype,
              '%entityid' => $activity->entityid,
              '%gid' => $activity->gid,
              '%activitycount' => $activity_count,
            ]);
        }
      }
    }
  }
}
